package MT::Plugin::BackgroundRebuild;
use strict;
use MT;
use MT::Plugin;
use MT::Template::Context;

use MT::Util qw(is_valid_email);
use MT::Util qw(remove_html);
use MT::I18N qw(encode_text);

###################################### Init Plugin #####################################

@MT::Plugin::BackgroundRebuild::ISA = qw(MT::Plugin);

my $plugin = new MT::Plugin::BackgroundRebuild({
	name => 'Background Rebuilder',
	version => '1.0RC2(r2)',
	description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
	author_name => 'Junnama Noda',
	author_link => 'http://junnama.alfasado.net/online/',
	l10n_class => 'BackgroundRebuild::L10N',
	settings => new MT::PluginSettings([
		['get_result', { Default => 0 }],
		['result_by_mail', { Default => 0 }],
		['mail_to', { Default => '' }],
		['ajax_start', { Default => 3000 }],
		['ajax_period', { Default => 1000 }],
		['ajax_limit', { Default => 180 }],
		['rebuild_period', { Default => 0 }],
	]),
	system_config_template => 'BackgroundRebuild.tmpl',
});

MT->add_plugin($plugin);

####################################### Callbacks ######################################

MT->add_callback('MT::App::CMS::AppTemplateSource.rebuild_confirm', 10, $plugin, \&_add_rebuilder);
MT->add_callback('MT::App::CMS::AppTemplateSource.blog-left-nav', 10, $plugin, \&_resize_rebuilder);
MT->add_callback('CMSPreSave_entry', 10, $plugin, \&_pre_save);
MT->add_callback('CMSPostSave.entry', 10, $plugin, \&_post_save);
MT->add_callback('CMSPostDelete_entry', 9, $plugin, \&_delete_entry);
MT->add_callback('MT::App::CMS::AppTemplateSource.rebuilding', 6, $plugin, \&_rebuild_selected);
MT->add_callback('MT::App::CMS::AppTemplateSource.itemset_action_widget', 1, $plugin, \&_replace_plugin_action);
MT->add_callback('MT::App::CMS::AppTemplateSource.entry_actions', 10, $plugin, \&_replace_delete_action);
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_entry', 10, $plugin, \&_embed_category);
MT->add_callback('MT::App::CMS::AppTemplateSource.rebuild-stub', 10, $plugin, \&_resize_rebuilder);
MT->add_callback('MT::App::CMS::AppTemplateSource.edit_entry', 6, $plugin, \&_saved_and_rebuild);

######################################### Main #########################################

sub _saved_and_rebuild {
	my ($eh, $app, $tmpl_str) = @_;
	my $query = $app->param;
	my $blog_id = $app->blog->id;
	my $saved_changes = $query->param('saved_changes');
	my $saved_added = $query->param('saved_added');
	my $bgrb = $query->param('bgrb');
	my $entry_id = $query->param('id');
	if ((($saved_changes) || ($saved_added)) && (! $bgrb)) {
		my $return_url = $app->base.$app->uri.'?'.$app->query_string.';bgrb=1';
		$return_url =~ s/\;/&/g;
		&_rebuild($blog_id, '', $return_url, $entry_id, '');
	}
}

sub _embed_category {
	my ($eh, $app, $tmpl_str) = @_;
	my $query = $app->param;
	my $entry_id = $query->param('id');
	if ($entry_id ne '') {
		my $iter = MT::Placement->load_iter({
		entry_id=>$entry_id,
		});
		my $str;
		if (defined $iter) {
			while (my $placement = $iter->()) {
				my $cat_id = $placement->category_id;
				$str .= '<input type="hidden" name="add_category_id_'.$cat_id;
				$str .= '" id="add_category_id_'.$cat_id.'" />'."\n";
				if ($placement->is_primary == 1) {
					$str .= '<input type="hidden" name="category_id';
					$str .= '" id="category_id" value="'.$cat_id.'" />'."\n";
				}
			}
		}
		my $old = '<div id="entry-container">';
		$old = quotemeta($old);
		my $new =<<"HTML";
<div id="entry-container">
<TMPL_IF NAME=DISP_PREFS_SHOW_CATEGORY>
<TMPL_ELSE>
$str
</TMPL_IF>
HTML
		$$tmpl_str =~ s/$old/$new/;
	}
}

sub _rebuild_selected {
	my ($eh, $app, $tmpl_str) = @_;
	my $query = $app->param;
	my $mode = $query->param('__mode');
	if ($mode ne 'rebuild_new_phase') {
		return 1;
	}
	my @ids = $query->param('id');
	my $alt_publish = $query->param('alt_publish');
	my $mt_path = $app->base.$app->uri();
	my $blog_id = $query->param('blog_id');
	my $query_str;
	if ($alt_publish != 3) {
		$query_str = $app->uri_params(mode => 'list_entries',
		args => {	'_type' => 'entry',
					'tab' => 'entry',
					'blog_id' => $blog_id,
		});
	} else {
		$query_str = $app->uri_params(mode => 'list_entries',
		args => {	'_type' => 'entry',
				'blog_id' => $blog_id,
				'saved_deleted' => 1,
		});
	}
	my $return_path = $mt_path.$query_str;
	if (($alt_publish == 1) || ($alt_publish == 2) || ($alt_publish == 3)) {
		my $delete_file = MT->config('DeleteFilesAtRebuild');
		my $blog; my $site_path; my $site_url;
		foreach my $entry_id ( @ids ) {
			my $entry = MT::Entry->load({ id => $entry_id });
			if (MT->config('DeleteFilesAtRebuild')) {
				if ((($alt_publish == 1) && ($delete_file == 1)) || ($alt_publish == 3)) {
					$app->publisher->remove_entry_archive_file(Entry => $entry, ArchiveType => 'Individual');
				}
			}
			if ($alt_publish != 3) {
				$entry->status($alt_publish);
				$entry->save;
			} else {
				$entry->remove;
			}
		}
	}
	if ($alt_publish == 1) {
		&_rebuild($blog_id, 'Monthly,Category,index', $return_path, '', '');
		return 0;
	} elsif (($alt_publish == 2) || ($alt_publish == 0)) {
		&_rebuild($blog_id, 'Entries', $return_path, 0, @ids);
		return 0;
	} elsif ($alt_publish == 3) {
		&_rebuild($blog_id, 'Individual,Monthly,Category,index', $return_path, 0, '');
		return 0;
	}
}

sub _replace_delete_action {
	my ($eh, $app, $tmpl_str) = @_;
	my $old = quotemeta('<input type="button" onclick="doRemoveItems(this.form, \'<MT_TRANS phrase="entry">\', \'<MT_TRANS phrase="entries">\')"');
	my 	$new = '<input type="button" onclick="';
	$new .= 'var count = countMarked(this.form, \'\'); if (!count) {';
	my $msg = $plugin->translate('You did not select any Entries.');
	$new .= 'alert(\''.$msg.'\');return false;} else {';
	$msg = $plugin->translate('Are you sure you want to delete the ');
	$new .= 'if (confirm(\''.$msg;
	$msg = $plugin->translate('selected Entries?');
	$new .= '\' + count + \''.$msg.'\')) {} else {return false;} }';
	$new .= 'document.getElementById(\'alt_publish\').value = 3;';
	$new .= 'doForMarkedInThisWindow(this.form, \'<MT_TRANS phrase="entry">\', \'';
	$new .= '<MT_TRANS phrase="entries">\', null, \'rebuild_new_phase\', {<TMPL_IF NAME=BLOG_ID>\'blog_id\': ';
	$new .= '\'<TMPL_VAR NAME=BLOG_ID>\'</TMPL_IF>}, \'<MT_TRANS phrase="to rebuild">\');';
	$new .= 'document.getElementById(\'alt_publish\').value = 0;';
	$new .= 'return false"';
	$$tmpl_str =~ s/$old/$new/;
}

sub _replace_plugin_action {
	my ($eh, $app, $tmpl_str) = @_;
	my $old = quotemeta('<select name="plugin_action_selector"');
	my $new = '<select id="plugin_action_selector" name="plugin_action_selector"';
	$$tmpl_str =~ s/$old/$new/;
	$old = quotemeta('<input type="button" onclick="doPluginAction');
	$new = '<input type="button" onclick="';
	$new .= 'var count = countMarked(this.form, \'\'); if (!count) {';
	my $msg = $plugin->translate('You did not select any Entries.');
	$new .= 'alert(\''.$msg.'\');return false;} else {} ';
	$new .= 'if (document.getElementById(\'plugin_action_selector\').value==\'set_published\'){';
	$new .= 'document.getElementById(\'alt_publish\').value = 2;';
	$new .= 'doForMarkedInThisWindow(this.form, \'<MT_TRANS phrase="entry">\', \'';
	$new .= '<MT_TRANS phrase="entries">\', null, \'rebuild_new_phase\', {<TMPL_IF NAME=BLOG_ID>\'blog_id\': ';
	$new .= '\'<TMPL_VAR NAME=BLOG_ID>\'</TMPL_IF>}, \'<MT_TRANS phrase="to rebuild">\');';
	$new .= 'document.getElementById(\'alt_publish\').value = 0;';
	$new .= 'return false};';
	$new .= 'if (document.getElementById(\'plugin_action_selector\').value==\'set_draft\'){';
	$new .= 'document.getElementById(\'alt_publish\').value = 1;';
	$new .= 'doForMarkedInThisWindow(this.form, \'<MT_TRANS phrase="entry">\', \'';
	$new .= '<MT_TRANS phrase="entries">\', null, \'rebuild_new_phase\', {<TMPL_IF NAME=BLOG_ID>\'blog_id\': ';
	$new .= '\'<TMPL_VAR NAME=BLOG_ID>\'</TMPL_IF>}, \'<MT_TRANS phrase="to rebuild">\');';
	$new .= 'document.getElementById(\'alt_publish\').value = 0;';
	$new .= 'return false};';
	$new .= 'doPluginAction';
	$$tmpl_str =~ s/$old/$new/;
	$$tmpl_str .= '<input type="hidden" name="alt_publish" id="alt_publish" value="0" />';
}

sub _pre_save {
	my ($eh, $app, $obj, $original) = @_;
	my $old_file;
	if (MT->config('DeleteFilesAtRebuild')) {
		use CGI;
		my $query = new CGI;
		my $category_id = $query->param('category_id');
		my $blog = $app->blog;
		my $blog_id = $blog->id;
		if (defined $original) {
			my $entry_cat = $original->category;
			my $cat_id;
			if (defined $entry_cat) {
				$cat_id = $entry_cat->id;
			}
			if ($cat_id != $category_id) {
				$old_file = &get_entry_path($blog, $original->permalink);
				use MT::TemplateMap;
				my $map = MT::TemplateMap->load({
							blog_id => $blog_id,
							archive_type => 'Individual',
							is_preferred => 1,
							});
				if (defined $map) {
					my $file_template = $map->file_template;
					if ($file_template =~ /%.{0,1}c/) {
						if (-f $old_file) {
							unlink $old_file;
						}
					}
				}
			}
		}
	}
	return 1;
}

sub _post_save {
	my ($eh, $app, $obj, $original) = @_;
	use CGI;
	my $query = new CGI;
	my $category_id = $query->param('category_id');
	my $blog_id = $obj->blog_id;
	use MT::Placement;
	my @entry_cats = MT::Placement->load({
										entry_id=>$obj->id, });
	my @categories = ();
	my @names = $query->param;
	foreach my $name (@names) {
		if ($name =~ /add_category_id_[0-9]{1,}/) {
			my $add_cat = $1 if {$name =~ /add_category_id_([0-9]{1,})/};
			if ($category_id != $add_cat) {
				push (@categories, $add_cat);
			}
		}
	}
	if (defined @entry_cats) {
		my $exist_cat = 0;
		foreach my $place (@entry_cats) {			my $place_id = $place->id;
			if ($place_id != $category_id) {
				if (grep (/^$place_id$/, @categories)) {
					if ($place->is_primary == 1) {						$place->is_primary(0);
						$place->save or die $place->errstr;
						@categories = grep (!/^$place_id$/, @categories)
					}
				} else {
					$place->remove or die $place->errstr;
				}
			} else {
				$exist_cat = 1;
				$place->is_primary(1);
				$place->save or die $place->errstr;
			}
		}
		unless ($exist_cat) {
			&new_placement($obj, $category_id, 1);
		}
	}
	foreach my $new_place (@categories) {
		&new_placement($obj, $new_place, 0);
	}
	my $entry_id = $obj->id;
	my $mt_path = $app->base.$app->uri();
	my $to_ping_urls = $query->param('to_ping_urls');
	my $blog = $app->blog;
	my $query_str;
	if (defined $original) {
		$query_str = $app->uri_params(mode => 'view',
		args => {	'_type' => 'entry',
					'id' => $entry_id,
					'blog_id' => $blog_id,
					'saved_changes' => 1,
		});
	}else{
		$query_str = $app->uri_params(mode => 'view',
		args => {	'_type' => 'entry',
					'id' => $entry_id,
					'blog_id' => $blog_id,
					'saved_added' => 1,
		});
    }
	my $return_path = $mt_path.$query_str;
	if ($obj->status == 2) {
		if ($to_ping_urls) {
			use MT::WeblogPublisher; 
			my $pub = MT::WeblogPublisher->new;
			$pub->rebuild_entry(
				BlogID => $blog_id,
				Entry => $obj,
				NoIndexes => 1,
			);
		}
		my $mt = MT->instance;
		my $old_status; my $atom_id;
		if (defined $original) {
			$old_status = $original->status;
			$atom_id = $original->atom_id;
		} else {
			$old_status = 0;
		}
		if (($to_ping_urls) || (! $atom_id)) {
			my $results = $mt->ping_and_save(Blog => $blog, Entry => $obj,
									OldStatus => $old_status) or return 1;
			my $has_errors = 0;
			require MT::Log;
			for my $res (@$results) {
			$has_errors++, $app->log({
					message => $app->translate("Ping '[_1]' failed: [_2]", $res->{url}, encode_text($res->{error}, undef, undef)),
					class => 'system',
					level => MT::Log::WARNING()
					}) unless $res->{good};
			}
			if ($has_errors) {
				$return_path .= '&ping_errors=1';
			}
		}
	}

	my $mt_path = 'Location: '.$return_path."\n\n";
	print $mt_path;
	return 0;
}

sub _delete_entry {
	my ($eh, $app, $obj) = @_;
	my $alt_obj = $obj->previous;
	if (!defined $alt_obj) {
		$alt_obj = $obj->next;
	}
	my $entry_id = $alt_obj->id;
	my $mt_path = $app->base.$app->uri();
	my $blog_id = $obj->blog_id;
	my $query_str = $app->uri_params(mode => 'list_entries',
	args => {	'_type' => 'entry',
				'blog_id' => $blog_id,
				'saved_deleted' => 1,
	});
	my $return_path = $mt_path.$query_str;
	my $iter = MT::Placement->load_iter({
	entry_id=>$obj->id
	});
	while (my $placement = $iter->()) {
		$placement->remove;
	}
	if (MT->config('DeleteFilesAtRebuild')) {
		$app->publisher->remove_entry_archive_file(Entry => $obj, ArchiveType => 'Individual');
	}
	&_rebuild($blog_id, 'Individual', $return_path, $entry_id, '');
	return 0;
}

sub _add_rebuilder {
	my ($eh, $app, $tmpl_str) = @_;
	my $query = $app->param;
	my $blog_id = $query->param('blog_id');
	my $process_id = $query->param('process_id');
	my $background = $query->param('background');
	my $type = $query->param('type');
	my $blog_list = $query->param('blog_list');
	my $tmp_list = $blog_list;
	my $return_blog = $query->param('return_blog');
	my $mt_base = $app->base.$app->uri();
	my $old = quotemeta('<select name="type">');
	my $new = '<select name="type" id="type">';
	$$tmpl_str =~ s/$old/$new/;
	$old = quotemeta('</form>');
	my $bg_rebuild = $plugin->translate('Background Rebuild');
	my $rebuild_all = $plugin->translate('Rebuild All Blogs');
	require MT::Blog;
	my $iter = MT::Blog->load_iter;
	my $blogs = '';
	while ( my $blog = $iter->() ) {
		$blogs .= $blog->id.',';
	}
	$blogs =~ s/(.{1,})\,$/$1/;
	$new = <<"HTML";

<hr />
<input type="button" value="$bg_rebuild" onclick="submitForm()" />
<input type="button" value="$rebuild_all" onclick="rebuildAll()" />
</form>
<script type="text/javascript">
	function submitForm() {
		var typeSelect = document.getElementById('type').value;
		var mtUrl = "$mt_base";
		mtUrl = mtUrl+"?__mode=rebuild_confirm&blog_id=$blog_id&background=1&type=";
		mtUrl = mtUrl+typeSelect;
		window.location.href = mtUrl;
	}
	function rebuildAll() {
		var mtUrl = "$mt_base";
		mtUrl = mtUrl+"?__mode=rebuild_confirm&blog_id=$blog_id&background=1&blog_list=$blogs&return_blog=$blog_id";
		window.location.href = mtUrl;
	}
</script>
HTML
	$$tmpl_str =~ s/$old/$new/g;
	my $view_site = '';
	my $rebuilt_msg = $plugin->translate('Rebuilding process is running.');
	if ($blog_id ne '0') {
		$view_site = '<a target="_blank" href="<TMPL_VAR NAME=BLOG_URL ESCAPE=HTML>"><MT_TRANS phrase="View your site">.</a>';
	}
	if ($background ne '0') {
		if ($process_id eq '') {
			$process_id = $app->make_magic_token;
		}else{
			$process_id .= ','.$app->make_magic_token;
		}
	}
	my $get_result = $plugin->get_config_value('get_result');
	if ($get_result == 1) {
		$view_site .= "\n".'<input type="hidden" id="process_ids" value="'.$process_id.'" />';
		$view_site .= "\n".'<ul id="text" style="margin-bottom:0;font-size:x-small"></ul>'."\n";
		my $app_path = $app->base.$app->path;
		$app_path .= 'plugins/BackgroundRebuild/BackgroundRebuild.cgi?__mode=BackgroundRebuild&process_id=';
		my $msg_begin = $plugin->translate('Blog');
		my $msg_end = $plugin->translate(' has been rebuilt.');
		my $ajax_err = $plugin->translate('Can\'t get result of Rebuilt.');
		my $ajax_period = $plugin->get_config_value('ajax_period');
		my $ajax_start = $plugin->get_config_value('ajax_start');
		my $ajax_limit = $plugin->get_config_value('ajax_limit');
		my $taken = $plugin->translate('Taken:');
		my $sec_str = $plugin->translate('sec.');
		$view_site .= &ajax_check_init($ajax_start, $ajax_period, $ajax_limit, $ajax_err, $msg_begin, $msg_end, $app_path, $taken, $sec_str);
		$view_site .= &ajax_check;
	}
	if (($background == 1) && ($type ne '')) {
		my $return_path = $mt_base.'?__mode=rebuild_confirm&blog_id='.$blog_id.'&background=0';
		$return_path .= '&process_id='.$process_id;
		&_rebuild($blog_id,$type,$return_path, '', '');
	}elsif ((($background == 1) && ($type eq '')) && ($blog_list ne '')) {
		my @blog_ids = split(/\,/,$blog_list);
		$blog_id = $blog_ids[0];
		$blog_list = '';
		my $count = @blog_ids;
		if ($count > 1) {
			$count--;
			for (1..$count) {
				$blog_list .= $blog_ids[$_].',';
			}
			$blog_list =~ s/(.{1,})\,$/$1/;
		}
		my $rebuild_period = $plugin->get_config_value('rebuild_period');
		if (($rebuild_period != 0) && ($blogs ne $tmp_list)) {
			sleep $rebuild_period;
		}
		my $return_path;
		if ($blog_list eq '') {
			$return_path = $mt_base.'?__mode=rebuild_confirm&blog_id='.$return_blog.'&background=';
			$return_path .= '0';
		}else{
			$return_path = $mt_base.'?__mode=rebuild_confirm&blog_id='.$blog_id.'&background=';
			$return_path .= '1';
		}
		$return_path .= '&blog_list='.$blog_list;
		$return_path .= '&return_blog='.$return_blog;
		$return_path .= '&process_id='.$process_id;
		&_rebuild($blog_id, 'Individual,Monthly,Category,index', $return_path, 0, '');
	}
	if ($background eq '0') {
		$old = quotemeta('<h2><span class="weblog-title-highlight"><TMPL_VAR NAME=BLOG_NAME ESCAPE=HTML>:</span> <MT_TRANS phrase="Rebuild"></h2>');
		$new = <<"HTML";
<h2><span class="weblog-title-highlight"><TMPL_VAR NAME=BLOG_NAME ESCAPE=HTML>:</span> <MT_TRANS phrase="Rebuild"></h2>
<h4 class="message">
$rebuilt_msg
$view_site
</h4>
HTML
	$$tmpl_str =~ s/$old/$new/;
	}
}

sub _resize_rebuilder {
	my ($eh, $app, $tmpl_str) = @_;
	my $old = quotemeta('width=400,height=300');
	my $new = 'width=400,height=400';
	$$tmpl_str =~ s/$old/$new/;
	my $mt_base = $app->base.$app->uri();
	$old = quotemeta('<input type="button" onclick="doRebuild(\'<TMPL_VAR NAME=BLOG_ID>\')"');
	my $ev = 'window.open(\''.$mt_base.'?__mode=rebuild_confirm&amp;blog_id=1\', \'rebuild\', \'width=400,height=400,resizable=yes\')';
	$new = '<input type="button" onclick="'.$ev.';return false"';
	$$tmpl_str =~ s/$old/$new/;
}

######################################### subs #########################################

sub _rebuild {
	my ($blog_id, $archive_type, $return_path, $entry_id, @entries) = @_;
	my $mt_path = 'Location: '.$return_path."\n\n";
	print $mt_path;
	my $process_id = $1 if {$return_path =~ /process_id=.{1,}\,(.{1,}$)/s};
	if ($process_id eq '') {
		$process_id = $1 if {$return_path =~ /process_id=(.{1,}$)/s};
	}
	my $pid;
	$| = 1;
	FORK: {
		if( $pid = fork ) {
			close(STDOUT);
			wait;
		} elsif (defined $pid) {
			close(STDOUT);
			&do_rebuild($blog_id, $archive_type, $process_id, $entry_id, @entries);
			exit;
		} elsif ( $! =~ /No more process/) {
			sleep 2;
			redo FORK;
		} else {
			die "Can't fork: $\n";
		}
	}
}

sub do_rebuild {
	my ($blog_id, $archive_type, $process_id, $entry_id, @entries) = @_;
	my $mt_dir;
	if ( $0 =~ m!(.{1,}[/\\])! ) {
		$mt_dir = $1;
	}else{
		$mt_dir = './';
	}
	unshift @INC, $mt_dir . '../../lib';
	unshift @INC, $mt_dir . '../../extlib';
	use MT;
	my $mt = MT->new(Config => $mt_dir . '../../mt-config.cgi');
	if ($archive_type ne 'Entries') {
		if ($entry_id == 0) {
			my $start_time = time;
			if ($archive_type =~ /index/) {
				$archive_type =~ s/index//;
				$archive_type =~ s/^\,//;
				$archive_type =~ s/\,$//;
				$archive_type = 'index,'.$archive_type;
			} 
			my @archive_types = split(/\,/,$archive_type);
			foreach my $type ( @archive_types ) {
				if ($type eq 'index') {
					$mt->rebuild_indexes(
						BlogID => $blog_id,
					);
				}else{
					$mt->rebuild(
						BlogID => $blog_id,
						ArchiveType => $type,
						NoIndexes => 1,
					);
				}
			}
			my $get_result = $plugin->get_config_value('get_result');
			my $result_by_mail = $plugin->get_config_value('result_by_mail');
			if (($get_result == 1) || ($result_by_mail == 1)) {
				use MT::Blog;
				my $blog = MT::Blog->load($blog_id);
				my $end_time = time;
				$end_time = $end_time - $start_time; 
				if ($get_result == 1) {
					$process_id =~ s/\.//; $process_id =~ s/\///;
					$process_id = './plugins/BackgroundRebuild/tmp/'.$process_id;
					open(OUT, ">$process_id");
					print OUT 'Rebuild is done.'.','.$blog_id.','.$blog->site_url.','.$end_time;
					close(OUT);
				}
				if ($result_by_mail == 1) {
					my $to = $plugin->get_config_value('mail_to');
					if (&is_valid_email($to)) {
						my $msg_begin = $plugin->translate('Blog');
						my $msg_end = $plugin->translate(' has been rebuilt.');
						my $taken = $plugin->translate('Taken:');
						my $sec_str = $plugin->translate('sec.');
						my $blog_name = $blog->name;
						my $subject = '['.$blog_name.']'.$msg_begin.$msg_end;
						my $body = $msg_begin.'['.$blog_name.']'.$msg_end."\n\n".$taken.$end_time.$sec_str;
						my $view_site = $plugin->translate('View your site');
						my $version = $plugin->version; my $name = $plugin->name;
						my $author_name = $plugin->author_name; my $author_link = $plugin->author_link;
						my $from = $name.' <'.$to.'>';
						my $ver_str = $plugin->translate('Ver: ');
						my $footer = "\n\n--\nPowered by ".$name."\n".$ver_str.$version;
						$footer .= "\n".$author_link;
						$body .= "\n\n".$view_site." => ".$blog->site_url.$footer;
						&send_mail($from, $to, $subject, $body);
					}
				}
			}
		}else{
			$mt->rebuild_indexes(
					BlogID => $blog_id,
			);
			$mt->rebuild_entry(
				BlogID => $blog_id,
				Entry => $entry_id,
				BuildDependencies => 1,
				NoIndexes => 1,
			);
			my $entry = MT::Entry->load($entry_id);
			my $prev = $entry->previous;
			my $next = $entry->next;
			if (defined $prev){
				$mt->rebuild_entry(
					BlogID => $blog_id,
					Entry => $prev->id,
					NoIndexes => 1,
				);
			}
			if (defined $next){
				$mt->rebuild_entry(
					BlogID => $blog_id,
					Entry => $next->id,
					NoIndexes => 1,
				);
			}
			my $delete_file = MT->config('DeleteFilesAtRebuild');
			if (($entry->status == 1) && ($delete_file == 1)) {
				my $path = $entry->permalink;
				require MT::Blog;
				my $blog = MT::Blog->load($blog_id);
				my $site_url = $blog->site_url;
				$site_url =~ s/\/$//;
				$site_url = quotemeta($site_url);
				my $site_path = $blog->site_path;
				$site_path =~ s/\/$//;
				$path =~ s/$site_url/$site_path/; 
				if (-e $path) {
					unlink($path);
				}
			}
		}
	}else{
		$mt->rebuild_indexes(
			BlogID => $blog_id,
		);
		foreach my $entry ( @entries ) {
			$mt->rebuild_entry(
				BlogID => $blog_id,
				Entry => $entry,
				NoIndexes => 1,
			);
		}
	}
}

sub get_new_entry_id {
	my ($blog_id, $user_id) = @_;
	my $entry = MT::Entry->new;
	$entry->blog_id($blog_id);
	$entry->author_id($user_id);
	$entry->status(1);
	$entry->save or die $entry->errstr;
	my $entry_id = $entry->id;
	$entry->remove;
	return $entry_id;
}

sub new_placement {
	my ($obj, $category_id, $is_primary) = @_;
	my $placement = MT::Placement->new;
	$placement->blog_id($obj->blog_id);
	$placement->entry_id($obj->id);
	$placement->category_id($category_id);
	$placement->is_primary($is_primary);
	$placement->save or die $placement->errstr;
	return $placement;
}

sub send_mail {
	my ($from, $to, $subject, $body) = @_;
	my %head = (	To => $to, from => $from, 
					Subject => $subject );
	my $charset = MT->config('PublishCharset');
	my $mail_enc = uc(MT->config('MailEncoding') || $charset);
	$head{'Content-Type'} = qq(text/plain; charset="$mail_enc");
	use MT::Mail;
	MT::Mail->send(\%head, $body) or
	return (0, MT->translate(
	"The error occurred.", MT::Mail->errstr));
}

sub get_entry_path {
	my ($blog, $permalink) = @_;
	my $arcive_path = $blog->archive_path;
	$arcive_path =~ s/\/$//;
	my $archive_url = $blog->archive_url;
	$archive_url =~ s/\/$//;
	$archive_url = quotemeta($archive_url);
	$permalink =~ s/^$archive_url/$arcive_path/;
	return $permalink;
}

######################################### Ajax #########################################

sub ajax_check_init {
	my ($ajax_start, $ajax_period, $ajax_limit, $ajax_err, $blog_str, $rebuilt_str, $app_path, $taken, $sec_str) = @_;
	return <<"HTML";
<script type="text/javascript">
	var sTimer = setTimeout('ajaxStart()', $ajax_start);
	var gTimer;
	function ajaxStart(){
		gTimer = setInterval('checkRes()', $ajax_period);
		clearTimeout(sTimer);
	}
	var tLimit = $ajax_limit;
	var errMsg = "$ajax_err";
	var blogStr = "$blog_str";
	var rebuiltStr = "$rebuilt_str";
	var appPath = "$app_path";
	var takenTime = "$taken";
	var secStr = "$sec_str";
HTML
}

sub ajax_check {
	return <<'HTML';
	var gCount = 0;
	var xmlhttp
	/*@cc_on @*/
	/*@if (@_jscript_version >= 5)
	try {
		xmlhttp=new ActiveXObject("Msxml2.XMLHTTP");
		}catch(e){
			try{
				xmlhttp=new ActiveXObject("Microsoft.XMLHTTP");
			}catch (err) {
				xmlhttp=false;
			}
	}
	@else
		xmlhttp=false
	@end @*/
	if (!xmlhttp && typeof XMLHttpRequest!='undefined') {
		try {
			xmlhttp = new XMLHttpRequest();
		} catch (e) {
			xmlhttp=false;
		}
	}
	if (!xmlhttp && window.createRequest) {
		try {
			xmlhttp = window.createRequest();
		} catch (e) {
			xmlhttp=false;
		}
	}
	function getNext(path_to_get, process_id) {
		var process_fld = document.getElementById('process_ids');
		if (xmlhttp) {
			xmlhttp.open("GET", path_to_get ,true);
			xmlhttp.onreadystatechange=function() {
				if (xmlhttp.readyState==4) {
					var str = xmlhttp.responseText;
					var strArray = str.split(",");
					if (strArray[0] == 'Rebuild is done.') {
						var msgfld = document.getElementById('text');
						var msg = '<li style="margin-top:0;margin-bottom:0;"><a target="_blank" href="';
						msg = msg + strArray[2] + '">' + blogStr + strArray[1] + rebuiltStr;
						msg = msg + '(' + takenTime + strArray[3] + secStr + ')</a></li>';
						msgfld.innerHTML = msgfld.innerHTML + msg;
						str = process_fld.value.replace(process_id, '');
						str = str.replace(/^\,/,'');
						str = str.replace(/\,$/,'');
						str = str.replace(/\,\,/,'');
						process_fld.value = str;
						if (str == '') {
							clearInterval(gTimer);
						}
					}
				}
			}
		xmlhttp.send(null);
		}
	}
	function checkRes() {
		var processes = document.getElementById('process_ids').value;
		processes = processes.split(",");
		var reqestpth = appPath;
		reqestpth = reqestpth + processes[0];
		gCount++;
		if (gCount < tLimit) { 
			getNext(reqestpth, processes[0]);
		}else{
			clearInterval(gTimer);
			var msgfld = document.getElementById('text');
			msgfld.innerHTML = msgfld.innerHTML + '<li style="color:red;">' + errMsg + '</li>';
		}
	}
</script>
HTML
}

1;